#!/bin/bash

# Script to patch in support for Intel i945GME
# Graphics card not recognized as i945 by kernel
# in default Ubuntu 8.04 because 945GM is 0x27A2 but 945GME is 0x27AE
# this is for Launchpad Bug #204762
# 
# Script by Mackenzie Morgan (maco.m@ubuntu.com)
# No guarantee, no warranty...but it *should* work

echo "Here's what will happen:"
echo "1. Check if you even need this"
echo "2. Download kernel sources in $HOME/src"
echo "3. Modify kernel sources to include the definition for Intel i945GME"
echo "4. Compile the kernel"
echo "5. Install the kernel"

# Check to see if they have an i945GME
VGADEV=$(lspci -vn | grep VGA | awk '{ print $3 }')
if [ "$VGADEV" != "8086:27ae" ]
then
	echo "You don't need this.  Exiting."
	exit
fi

# go to ~/src/ ...if it doesn't exist, make it exist
cd ~
if [ -d src ] 
then
	cd src
else
	mkdir src
	cd src
fi

echo "================================"
echo "Now installing packages necessary for compiling kernel"
echo "================================"
sudo apt-get install linux-kernel-devel fakeroot build-essential
sudo apt-get build-dep linux-image-$(uname -r)
apt-get source linux-image-$(uname -r)

cd linux-2.6.24/

echo "================================"
echo "Updating architecture"
echo "================================"
chmod +x debian/scripts/misc/splitconfig.pl
debian/rules updateconfigs

echo "================================"
echo "Adding graphics card definition to kernel"
echo "================================"
# What this does:
# -#define IS_I945GM(dev) ((dev)->pci_device == 0x27A2)
# +#define IS_I945GM(dev) ((dev)->pci_device == 0x27A2 ||\
# +                       (dev)->pci_device == 0x27AE)
sed -ie 's/#define IS_I945GM(dev) ((dev)->pci_device == 0x27A2)/#define IS_I945GM(dev) ((dev)->pci_device == 0x27A2 || \\ \n                       (dev)->pci_device == 0x27AE)/' drivers/char/drm/i915_drv.h

# bump version so update manager isn't confused
sed -ie '1ilinux (2.6.24-16.30+1) hardy; urgency=low\n\n  * Make kernel aware that pci device 0x27AE is Intel i945GME\n\n -- Mackenzie Morgan <maco.m@ubuntu.com>  Tue, 29 Apr 2008 15:48:34 -0400\n' debian/changelog


echo "================================"
echo "Building the kernel"
echo "================================"
echo ""
FLAVOR=$(uname -r | awk -F- '{ print $3 }')
AUTOBUILD=1 NOEXTRAS=1 fakeroot debian/rules binary-$FLAVOR

# move back to  ~/src
cd ../

echo "================================"
echo "Done building kernel"
echo ""
echo "Installing new kernel"
echo "================================"
echo vesafb | sudo tee -a /etc/initramfs-tools/modules
echo fbcon | sudo tee -a /etc/initramfs-tools/modules
sudo dpkg -i linux*deb

echo "Please reboot for new kernel to take effect"
